
// 2016-02-27 JCT
//Moving the functions defined in the scope lib of fwBackup/fwBackupLauncher.pnl to this file

// Mapping of the user choices : state of the selections of each system and each manager in each system
// In each of the mapping element, we find the flag check in first position, then a list of manager for a system. We also add a list of log messages.
// Each manager is made of the flag check and the manager description. e.g. "WCCOActrl;xxx.ctl"

const int INDEX_SYSTEM_IS_CHECKED = 1;
const int INDEX_SYSTEM_LOG = 3;
// TODO ???





// Each element of the mapping is made of the thread ID (initialized to -1) and the thread state
const int THREAD_ID_INDEX = 1;
const int THREAD_STATE_INDEX = 2;
// Constants to indicate the state of each thread
const int THREAD_STOPPED = 0;
const int THREAD_RUNNING = 1;
const int THREAD_ERROR   = -1;


// System list table column
const string SYSTEM_LIST_NAME_COLUMN = "SYSTEM_NAME";
const string SYSTEM_LIST_BACKUP_PATH = "BCK_PATH";
const string SYSTEM_LIST_FREQUENCY = "FREQ";
const string SYSTEM_LIST_HOSTNAME = "HOSTNAME";
const string SYSTEM_LIST_PROJECT_NAME = "PROJECT_NAME";
const string SYSTEM_LIST_BACKUP_AGE = "AGE";
const string SYSTEM_LIST_NEXT_BACKUP = "NEXT_BACKUP";


// Separator of a manager type and its command
const string MANAGER_TYPE_COMMAND_SEPARATOR = ";";


// Texts to display for the state of the backup
const string BACKUP_STATE_NOT_READY = "Backup not ready.";
const string BACKUP_STATE_READY     = "Backup ready.";
const string BACKUP_STATE_RUNNING   = "Backup in progress.";
const string BACKUP_STATE_FINISHED  = "Backup finished.";
// Color of the backup state
const string BACKUP_COLOR_NOT_READY = "red";
const string BACKUP_COLOR_READY     = "grey";
const string BACKUP_COLOR_RUNNING   = "FwStateOKNotPhysics";
const string BACKUP_COLOR_FINISHED  = "green";


// Colors of the cell in the systems table depending on state
const string SYSTEM_BACKGROUND_COLOR_DEFAULT  = "white";
const string SYSTEM_BACKGROUND_COLOR_UNUSED   = "grey";
const string SYSTEM_BACKGROUND_COLOR_RUNNING  = "FwStateOKNotPhysics";
const string SYSTEM_BACKGROUND_COLOR_ERROR    = "red";
const string SYSTEM_BACKGROUND_COLOR_COMPLETE = "green";


// ------------------------------------------------------------------------------------------------

// Change the color of the cell for the given system
void changeSystemCellColor(string sSystem, string sColor)
{
  synchronized(SYSTEM_TABLE_MUTEX)
  {
    for (int i = 0 ; i < tableSystemList.lineCount() ; i++)
    {
      if (sSystem == tableSystemList.cellValueRC(i, SYSTEM_LIST_NAME_COLUMN))
      {
        tableSystemList.cellBackColRC(i, SYSTEM_LIST_NAME_COLUMN, sColor);        
      }    
    }
  }
}

// ------------------------------------------------------------------------------------------------

/* Display a log message in the appropriate system log window.
 If no system is specified the message is displayed in the global window (first)
 If flag logFile active, the message is written to a file
 */
void logMessage(string sMessage, string sSystemName = "", bool logFile = true)
{
  if (logFile && backupLogFile)
  {
    fputs(formatTime("%Y.%m.%d %H.%M.%S:",getCurrentTime(),"%03d") + "-INFO-" + (sSystemName == ""? getSystemName() : sSystemName) + "-" + sMessage + "\n", backupLogFile);
  }
    
  string sSystemName2 = sSystemName;
  if ( sSystemName == "")
  {
    sSystemName2 = getSystemName();
  }  
  
  sMessage = formatTime("%Y.%m.%d %H.%M.%S:",getCurrentTime(),"%03d") + " ["+sSystemName2+"] " + sMessage;
  
   
      
    synchronized(dsGlobalLog)
    {
      dynAppend(dsGlobalLog, sMessage);
    }
  
  
 
    displayLog(sSystemName);
    
 
  DebugTN(__FUNCTION__, sSystemName, sMessage);
}

// ------------------------------------------------------------------------------------------------

// Same as logMessage but displays a message in red for more visibility
void logError(string sMessage, string sSystemName = "")
{
  
  if (backupLogFile)
  {
    fputs(formatTime("%Y.%m.%d %H.%M.%S:",getCurrentTime(),"%03d") + "-ERROR-" + (sSystemName == ""? getSystemName() : sSystemName) + "-" + sMessage + "\n", backupLogFile);
  }
    
  synchronized (bErrorOccurred)
  {
    bErrorOccurred = true;
  }
  logMessage("<font color=#FF0000>" + sMessage + "</font>", sSystemName, false);  

}

// ------------------------------------------------------------------------------------------------

// Same as logError but in green
void logWarning(string sMessage, string sSystemName = "")
{
  if (backupLogFile)
  {
    fputs(formatTime("%Y.%m.%d %H.%M.%S:",getCurrentTime(),"%03d") + "-WARNING-" + (sSystemName == ""? getSystemName() : sSystemName) + "-" + sMessage + "\n", backupLogFile);
  }
  
  synchronized (bWarningOccurred)
  {
    bWarningOccurred = true;
  }
  logMessage("<font color=#FFA500>" + sMessage + "</font>", sSystemName, false);

}

// ------------------------------------------------------------------------------------------------

// Display the selected log
// Adds an icon to the currently selected tab and removes any icon from the other tabs
void displayLog(string sSystemName = "")
{
    
  
  

  sCurrentLogSystem = sSystemName;
    tabLogList.registerIcon(0, "logviewer_32x32.png");
    synchronized(dsGlobalLog)
    {
      for (int i = 1 ; i <= dynlen(dsGlobalLog) ; i ++)
      {
        textEditLog.append(dsGlobalLog[i]);
      }
      
      dynClear(dsGlobalLog);
    }
    
    

}

// ------------------------------------------------------------------------------------------------

// Resets the log
clearLog()
{
  textEditLog.text(""); 
}

// ------------------------------------------------------------------------------------------------

// Function to call at the end of a backup.
// It checks if other backups are still running, if not displays a pop up to inform the user
checkBackupsOver()
{
  synchronized(mThreadHasFinishedBackup)
  {
    bool bOneThreadIsRunning = false;
    for (int i = 1 ; i <=  mappinglen(mThreadHasFinishedBackup) ; i++)
    {
      if (mappingGetValue(mThreadHasFinishedBackup, i)[THREAD_STATE_INDEX] == THREAD_RUNNING)
      {
        bOneThreadIsRunning = true;        
      }      
    }
    
    if (!bOneThreadIsRunning)
    {
      infoArrow.backCol("unAlarm_Ok");
      textBackupState.backCol("unAlarm_Ok");
      infoArrow.visible(TRUE);
      textBackupState.visible(TRUE);
      textBackupState.text(BACKUP_STATE_FINISHED);
      enableUi(true);
      dyn_string dsExceptions;
      
      if (!bWarningOccurred && !bErrorOccurred)
      {
        logMessage("Backups finished on all systems.");
        unSendMessage_toAllUsers("Backups finished on all systems.", dsExceptions);
      }
      else if (bErrorOccurred)
      {
        logMessage("Backups finished with error(s).");
        unMessageText_send(
            getSystemName(),
            myManNum(),
            getSystemName() + "," + myManNum() + "," + getHostname() + "," + getUserName(),
            "user",
            "*",
            "ERROR",
            "Backups finished with error(s).",
            dsExceptions
        );										
      }
      else
      {
        logMessage("Backups finished with warning(s).");
        unMessageText_send(
            getSystemName(),
            myManNum(),
            getSystemName() + "," + myManNum() + "," + getHostname() + "," + getUserName(),
            "user",
            "*",
            "WARNING",
            "Backups finished with warning(s).",
            dsExceptions
        );						
      }
      
      // We need to check again before running a new backup
      pushButtonRun.enabled(false);
    }    
  }  
}

// ------------------------------------------------------------------------------------------------

// Enables or disables all widget in the UI, to prevent action while running the backups
enableUi(bool bEnable)
{
  pushButtonCheck.enabled(bEnable);
  pushButtonRun.enabled(bEnable);

  tableSystemList.enabled(bEnable);

  pushButtonSelectAllSystems.enabled(bEnable);

  pushButtonSelectNoSystem.enabled(bEnable);
  
  BACKUP_OPTIONS.enabled(bEnable);
}

// ------------------------------------------------------------------------------------------------

closePanel() 
{
  // If some thread are running, ask the user confirmation
  for (int i = 1 ; i <= mappinglen(mThreadHasFinishedBackup) ; i++)
  {
    synchronized(mThreadHasFinishedBackup)
    {
      if (mappingGetValue(mThreadHasFinishedBackup, i)[THREAD_STATE_INDEX] == THREAD_RUNNING)
      {
        dyn_float dfReturn;
        dyn_string dsReturn;
        ChildPanelOnCentralModalReturn(
            "vision/MessageWarning2", 
            "Backup running", 
            makeDynString(
                "$1:Backup is still running on system \"" + mappingGetKey(mThreadHasFinishedBackup,i) + "\", do you want to stop it ?",
                "$2:Yes",
                "$3:No"),
            dfReturn,
            dsReturn
          );
      
        if (dynlen(dfReturn) > 0 && dfReturn[1] == 1)
        {

          logMessage("Stopping thread " + mThreadHasFinishedBackup[mappingGetKey(mThreadHasFinishedBackup, i)][THREAD_ID_INDEX]);
          logMessage("State: " + mappingGetValue(mThreadHasFinishedBackup, i)[THREAD_STATE_INDEX]);

          stopThread(mThreadHasFinishedBackup[mappingGetKey(mThreadHasFinishedBackup, i)][THREAD_ID_INDEX]);
          mThreadHasFinishedBackup[mappingGetKey(mThreadHasFinishedBackup, i)][THREAD_STATE_INDEX] = THREAD_STOPPED;

          logMessage("State: " + mappingGetValue(mThreadHasFinishedBackup, i)[THREAD_STATE_INDEX]);
          logMessage("Backup on system \"" + mappingGetKey(mThreadHasFinishedBackup,i) + "\" stopped.");
        }
        else
        {
          return;
        }
      }
    }
  }
   
  PanelOff();
}


initializeTableSystems(dyn_string &dsExceptions)
{
  
  // Fill system list
  const int SEC_IN_HOUR = 3600;
  const int HOUR_IN_DAY = 24; 
  const string rootNfsPath = "/nfs/cs-tcr-backup2/backup/PVSS_BACKUP/";
  dyn_string dsSystemNames;
  dyn_uint diSystemIds; 
  dyn_string dsHostNames;
  dyn_int diPortNumbers;
  string backup_path, hostname, projname, backup_age;
  int iFrequency;
  int iSyncTime;
  int iWeekDay, iDay, iMonth, interval;
  dyn_string dsHostnameSplit;
  time t0 = getCurrentTime();
  
  getSystemNames(dsSystemNames,diSystemIds);
  for (int i = 1 ; i <= dynlen(dsSystemNames) ; i++)
  {
    // Add the ":" at the end of the system once and for all
    dsSystemNames[i] = dsSystemNames[i] + ":";
    
  dpGet(dsSystemNames[i] + "_DataManager.Backup.InputFile.Device", backup_path, dsSystemNames[i] + "_unApplication.hostname", hostname, dsSystemNames[i] + "_unApplication.projectName", projname, dsSystemNames[i] + "_OnlineBackup.interval", iFrequency,  dsSystemNames[i] + "_OnlineBackup.syncTime", iSyncTime, dsSystemNames[i] + "_OnlineBackup.syncWeekDay", iWeekDay, dsSystemNames[i] + "_OnlineBackup.syncDay", iDay, dsSystemNames[i] + "_OnlineBackup.syncMonth", iMonth, dsSystemNames[i] + "_OnlineBackup.interval", interval);
  iFrequency /= 3600;
  iFrequency /= 24;
  
  time nextSync = _vaSyncTimeBackup(interval, iSyncTime, iWeekDay, iDay, iMonth);
  string sNextSync = "";
 
  if (nextSync > 0 && interval != 0) {
    sNextSync = formatTime("%d.%m.%Y %H:%M", nextSync);
  }
  else {
    sNextSync = "---";
  }
  
  dsHostnameSplit=strsplit(hostname, ".");
  string nfsPath = rootNfsPath + dsHostnameSplit[1] + "/" + projname + "/" + projname + "/db/wincc_oa/lastval/lastflt.db";
  
  time fileTime;
  int elapsed = 0;
  bool fileExists = false;      
    if(access(nfsPath, R_OK) == 0)
      {
        fileExists = true;
        fileTime = getFileModificationTime(nfsPath);
        elapsed = (t0-fileTime);
        elapsed /= (24*3600);
        backup_age = elapsed + "d";
      }
    if(!fileExists) {
        backup_age = "NO BACKUP";
    }      
  
  
    mThreadHasFinishedBackup[dsSystemNames[i]] = makeDynAnytype(-1, THREAD_STOPPED);
       
    // Check the remote system is up to date and has the required DPE
    if (!dpExists(dsSystemNames[i] + UN_APPLICATION_DPNAME + ".backupConfiguration.managerList"))
    {      
      string sError = getCatStr("unBackup","1");
      strreplace(sError,"DP_NAME", UN_APPLICATION_DPNAME + ".backupConfiguration.managerList");
      strreplace(sError,"SYSTEM", dsSystemNames[i]);
      fwException_raise(
          dsExceptions,
          "ERROR",
          sError,
          "1"
      );
      logError(sError);
      
      // Ignore this system
      continue;      
    }
    
    //getting the WinCCOA version
    dyn_string dsWccoaPatches;
    string sWccoaVersion;
    dpGet(dsSystemNames[i]+"fwInstallationInfo.projectInfo.wccoaVersion", sWccoaVersion,
          dsSystemNames[i]+"fwInstallationInfo.projectInfo.wccoaPatchList",dsWccoaPatches);
    dynSort( dsWccoaPatches);
    
    // Fill the mapping
    mSystemConfiguration[dsSystemNames[i]] = makeDynAnytype();    
    mSystemConfiguration[dsSystemNames[i]][INDEX_SYSTEM_IS_CHECKED] = true;
    mSystemConfiguration[dsSystemNames[i]][INDEX_SYSTEM_LOG] = makeDynString();
    
    // Fill the table with the system list
    tableSystemList.appendLine(SYSTEM_LIST_NAME_COLUMN, dsSystemNames[i],
                               SYSTEM_LIST_BACKUP_PATH, backup_path,
                               SYSTEM_LIST_FREQUENCY, iFrequency,
                               SYSTEM_LIST_HOSTNAME, hostname,
                               SYSTEM_LIST_PROJECT_NAME, projname,
                               SYSTEM_LIST_BACKUP_AGE, backup_age,
                               SYSTEM_LIST_NEXT_BACKUP, sNextSync,
                               "WCCOA_VERSION", sWccoaVersion,
                               "WCCOA_PACTHES", (string)dsWccoaPatches);
  }  
  
//Try to change the color of all the null backup path cells in red, success on the test version, but not on the production system
/*  
if(backup_path != ""){
    for (int j = 0 ; j < tableSystemList.lineCount() ; j++)
    {
        tableSystemList.cellBackColRC(j, SYSTEM_LIST_NAME_COLUMN, SYSTEM_BACKGROUND_COLOR_ERROR);    
    }
  }
  else {}
*/
}


bool isBackupSequential()
{
  return BACKUP_OPTIONS.number == 0 ? true : false;
}





